/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2005 C.Marchand - christophe.marchand@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.common.config;

import java.util.Enumeration;
import java.util.Hashtable;

import org.xml.sax.SAXException;

import fr.gouv.finances.cp.utils.xml.marshal.InvalidXmlDefinition;
import fr.gouv.finances.cp.utils.xml.marshal.XmlAttributes;
import fr.gouv.finances.cp.utils.xml.marshal.XmlMarshallable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlOutputter;

/**
 * Represente la configuration personnelle de l'utilisateur
 * @author chm
 */
public class PersonnalConfigurationModel implements XmlMarshallable {
    public static final transient String TAG = "personnal-config";

    private Hashtable<String,SavedRequestsModel> savedRequests;
    
    public PersonnalConfigurationModel(String tagName) {
        super();
        savedRequests = new Hashtable<String,SavedRequestsModel>();
    }

    public void addCharacterData(String cData) throws SAXException {}
    public void addChild(XmlMarshallable child, String tagName) throws SAXException {
        if(SavedRequestsModel.TAG.equals(tagName)) {
            SavedRequestsModel srm = (SavedRequestsModel)child;
            savedRequests.put(srm.getKey(),srm);
        }
    }
    public XmlMarshallable getAttributes(XmlAttributes attributes) throws SAXException {
        return this;
    }
    public void marshall(XmlOutputter output) {
        output.startTag(TAG);
        for(Enumeration<SavedRequestsModel> enumer=savedRequests.elements();enumer.hasMoreElements();) {
            enumer.nextElement().marshall(output);
        }
        output.endTag(TAG);
    }
    public void validate() throws InvalidXmlDefinition {}
    @Override
    public Object clone() {
        // TODO
        return this;
    }
    public SavedRequestsModel getSavedRequests(ElementModel em) {
        String key = getKey(em);
        SavedRequestsModel srm = savedRequests.get(key);
        if(srm==null) {
            srm = new SavedRequestsModel(SavedRequestsModel.TAG,em);
            savedRequests.put(getKey(em),srm);
        }
        return srm;
    }
    
    private String getKey(ElementModel el) { return el.getParent().getParent().getId()+"|"+el.getParent().getId()+"|"+el.getId(); }
}
