/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2005 C.Marchand - christophe.marchand@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.common.config;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import fr.gouv.finances.cp.utils.xml.marshal.InvalidXmlDefinition;
import fr.gouv.finances.cp.utils.xml.marshal.NoeudModifiable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlAttributes;
import fr.gouv.finances.cp.utils.xml.marshal.XmlMarshallable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlOutputter;

/**
 * Represents a special-key-entry tag
 * @author chm
 */
public class SpecialKeyEntryModel implements NoeudModifiable, Cloneable {
	private static Logger logger = Logger.getLogger(SpecialKeyEntryModel.class);
	private NoeudModifiable _NMParent = null;
        private String configXPath = null;
	
    public static final transient String TAG = "special-key-entry";
    
    private boolean distinct=false, depends=false;
    private int pos=0;
    private XPathModel path;
    private TextModel key1, key2;
    private String id;

    public SpecialKeyEntryModel(String tagName) {
        super();
    }

    public void addCharacterData(String cData) throws SAXException {}
    public void addChild(XmlMarshallable child, String tagName) throws SAXException {
        if("path".equals(tagName)) {
            path=(XPathModel)child;
            path.setParentAsNoeudModifiable(this);
        } else if("key1".equals(tagName)) {
            key1 = (TextModel)child;
            key1.setParentAsNoeudModifiable(this);
        } else if("key2".equals(tagName)) {
            key2 = (TextModel)child;
            key2.setParentAsNoeudModifiable(this);
        }
    }

    public XmlMarshallable getAttributes(XmlAttributes attributes) throws SAXException {
        if(attributes.getValue("id")!=null)
            id = attributes.getValue("id");
    	if (attributes.getValue("pos")!=null)
    		pos = attributes.getIntValue("pos");
    	if (attributes.getValue("distinct")!=null)
    		distinct=attributes.getBooleanValue("distinct");
    	if (attributes.getValue("depends")!=null)
    		depends=attributes.getBooleanValue("depends");
        return this;
    }
    public void marshall(XmlOutputter output) {}
    public void validate() throws InvalidXmlDefinition {
        if(id==null || id.length()==0) throw new InvalidXmlDefinition("//"+TAG+"/@id is required ("+getParentAsNoeudModifiable().getConfigXPath()+"/"+TAG+")");
        NoeudModifiable localParent = getParentAsNoeudModifiable();
        while(!(localParent instanceof DocumentModel) && localParent!=null) localParent = localParent.getParentAsNoeudModifiable();
        if(localParent!=null) {
            DocumentModel dm = (DocumentModel)localParent;
            boolean found = false;
            for(SpecialKeyModel skm:dm.getSpecialKeys()) found |= skm.getId().equals(id);
            if(!found) throw new InvalidXmlDefinition(getConfigXPath()+" references an unknown special-key: "+id);
        }
    }

    @Override
    public SpecialKeyEntryModel clone() {
        SpecialKeyEntryModel other = new SpecialKeyEntryModel(TAG);
        other.depends=depends;
        other.distinct=distinct;
        try {other.addChild(this.path.clone(), "path");} catch (Throwable t) {}
//        other.path=path==null?null:path.clone();
        other.pos=pos;
        try {other.addChild(this.key1.clone(), "key1");} catch (Throwable t) {}
//        other.key1=(key1!=null?key1.clone():null);
        try {other.addChild(this.key2.clone(), "key2");} catch (Throwable t) {}
//        other.key2=(key2!=null?key2.clone():null);
        return other;
    }

    public boolean isDepends() {
        return depends;
    }
    public boolean isDistinct() {
        return distinct;
    }
    public XPathModel getPath() {
        return path;
    }
    public int getPos() {
        return pos;
    }
    public TextModel getKey1() { return key1; }
    public TextModel getKey2() { return key2; }

	public void modifyAttr(String attrName, String value) {
		// TODO Raccord de m�thode auto-g�n�r�
		
	}

	public void modifyAttrs(Attributes attrs) {
		try {
			getAttributes(new XmlAttributes(attrs));
		} catch (Exception e) {
			logger.error("Erreur lors de la mise � jour des attributs : "+e);
		}
	}

	public void setParentAsNoeudModifiable(NoeudModifiable p) {
		this._NMParent = p;
	}
	public NoeudModifiable getParentAsNoeudModifiable() {
		return this._NMParent;
	}
	public NoeudModifiable getChildAsNoeudModifiable(String tagName, String id) {
        if("path".equals(tagName)) {
            return path;
        } else if("key1".equals(tagName)) {
            return key1;
        } else if("key2".equals(tagName)) {
            return key2;
        }
	    else {
	    	return null;
	    }
	}

	public String[] getChildIdAttrName(String childTagName) {
		return null;
	}

	public void resetCharData() {
	}
        public String getId() { return id; }
    public String getIdValue() { return getId(); }
    public String getConfigXPath() {
        if(configXPath==null) {
            if(getParentAsNoeudModifiable()!=null) configXPath = getParentAsNoeudModifiable().getConfigXPath();
            else configXPath="";
            configXPath+="/"+TAG+"[@id='"+getId()+"']";
        }
        return configXPath;
    }
	
}
