/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2005 C.Marchand - christophe.marchand@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.common.config;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import fr.gouv.finances.cp.utils.xml.InvalidPathExpressionException;
import fr.gouv.finances.cp.utils.xml.dompath.DomPath;
import fr.gouv.finances.cp.utils.xml.marshal.InvalidXmlDefinition;
import fr.gouv.finances.cp.utils.xml.marshal.NoeudModifiable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlAttributes;
import fr.gouv.finances.cp.utils.xml.marshal.XmlMarshallable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlOutputter;

/**
 * Represente un chemin DomPath. Attention, plusieurs tags utilisent ce model
 * @author chm
 */
public class XPathModel implements NoeudModifiable {
    private static Logger logger = Logger.getLogger(XPathModel.class);
    private NoeudModifiable _NMParent = null;
    
    private String tag;
    private StringBuffer path;
    private DomPath domPath = null;
    
    private NoeudModifiable parent = null;
    
    public void setParent(NoeudModifiable parent) {
        this.parent = parent;
    }
    public XPathModel(String tagName) {
        super();
        this.tag=tagName;
        path = new StringBuffer();
    }
    public void addCharacterData(String cData) throws SAXException {
        path.append(cData);
    }
    public void addChild(XmlMarshallable child, String tagName)throws SAXException {}
    public XmlMarshallable getAttributes(XmlAttributes attributes)throws SAXException { return this; }
    public void marshall(XmlOutputter output) {
        output.startTag(tag);
        output.addCharacterData(path.toString());
        output.endTag(tag);
    }
    public void validate() throws InvalidXmlDefinition {
        if(getPath()==null || getPath().length()==0) throw new InvalidXmlDefinition("//"+getTag()+"/text() can not be empty");
    }
    public String getPath() {return path.toString();}
    public String getTag() {return tag;}
    public String getFormattedPath() {
        if(path==null) return null;
        return path.toString().replace('{','[').replace('}',']');
    }
    public DomPath getDomPath(javax.xml.namespace.NamespaceContext ns) throws InvalidPathExpressionException {
        if(domPath==null) {
            domPath = new DomPath(path.toString(),ns);
        }
        return domPath;
    }
    @Override
    public XPathModel clone() {
        XPathModel xp = new XPathModel(this.tag);
        xp.path = this.path;
        xp.domPath = this.domPath;
        return xp;
    }
    public NoeudModifiable getChildAsNoeudModifiable(String tagName, String id) {
        return null;
    }
    public void setParentAsNoeudModifiable(NoeudModifiable p) {
        this._NMParent = p;
    }
    public NoeudModifiable getParentAsNoeudModifiable() {
        return this._NMParent;
    }
    public void modifyAttr(String attrName, String value) {
        // TODO Raccord de m�thode auto-g�n�r�
        
    }
    public void modifyAttrs(Attributes attrs) {
        try {
            getAttributes(new XmlAttributes(attrs));
        } catch (Exception e) {
            logger.error("Erreur lors de la mise � jour des attributs : "+e);
        }
    }
    public String[] getChildIdAttrName(String childTagName) {
        return null;
    }
    public void resetCharData() {
        path.setLength(0);
    }
    
    public String getIdValue() { return null; }
    private String configXPath = null;
    public String getConfigXPath() {
        if(configXPath==null) {
            if(getParentAsNoeudModifiable()!=null) configXPath = getParentAsNoeudModifiable().getConfigXPath();
            else configXPath="";
            configXPath+="/"+tag;
        }
        return configXPath;
    }
    
}
