/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2007 JP.Tessier - jean-philippe.tessier@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.controls.models;

import java.util.Enumeration;
import java.util.Vector;

import org.xml.sax.SAXException;

import fr.gouv.finances.cp.utils.xml.marshal.InvalidXmlDefinition;
import fr.gouv.finances.cp.utils.xml.marshal.XmlAttributes;
import fr.gouv.finances.cp.utils.xml.marshal.XmlMarshallable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlOutputter;

/**
 * Modelise un ensemble de documents
 * @author chm
 */
public class ControlsModel implements XmlMarshallable {
	public static final transient String TAG = "controls";
	Vector<DocumentControlModel> documents;
	
	public ControlsModel(String tag) {
		super();
		documents = new Vector<DocumentControlModel>();
	}
	
	public void addCharacterData(String cData) throws SAXException {throw new SAXException("pas de CDATA attendu");}
	public void addChild(XmlMarshallable child, String tagName)	throws SAXException {
		if(DocumentControlModel.TAG.equals(tagName)) documents.add((DocumentControlModel)child);
		else throw new SAXException(tagName + ": tag non attendu");
	}
	public XmlMarshallable getAttributes(XmlAttributes attributes) throws SAXException {return this;}
	public void marshall(XmlOutputter output) {
            output.startTag(TAG);
            for(DocumentControlModel dm:documents) dm.marshall(output);
            output.endTag(TAG);
        }
	public void validate() throws InvalidXmlDefinition {
		for(DocumentControlModel dm:documents) dm.validate();
	}
	public Vector<DocumentControlModel> getDocuments() { return documents; }
	public DocumentControlModel getDocumentById(String docId) {
		for(Enumeration<DocumentControlModel> enumer=documents.elements();enumer.hasMoreElements();) {
			DocumentControlModel doc = enumer.nextElement();
			if(doc.getDocumentById().equals(docId)) return doc;
		}
		return null;
	}
	@Override
	public ControlsModel clone() {
		ControlsModel dom = new ControlsModel(TAG);
		for(DocumentControlModel dm:this.documents) dom.documents.add(dm);
		return dom;
	}
}
