/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2005 C.Marchand - christophe.marchand@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.updater.config;

import org.xml.sax.SAXException;

import fr.gouv.finances.cp.utils.xml.marshal.InvalidXmlDefinition;
import fr.gouv.finances.cp.utils.xml.marshal.XmlAttributes;
import fr.gouv.finances.cp.utils.xml.marshal.XmlMarshallable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlOutputter;

public class ComponentRefModel implements XmlMarshallable {
	public static final transient String TAG = "component-ref";
	private String id, version;

	public ComponentRefModel(String tag) {
		super();
	}

	public void addCharacterData(String cData) throws SAXException {
	}

	public void addChild(XmlMarshallable child, String tagName) throws SAXException {
	}

	public XmlMarshallable getAttributes(XmlAttributes attributes) throws SAXException {
		id=attributes.getValue("cid");
		version=attributes.getValue("version");
		return this;
	}

	public void marshall(XmlOutputter output) {
		output.startTag(TAG);
		output.addAttribute("cid",id);
		output.addAttribute("version",version);
		output.endTag(TAG);
	}

	public void validate() throws InvalidXmlDefinition {
		if(id==null) throw new InvalidXmlDefinition("cid attribute is mandatory for element "+TAG);
		if(version==null) throw new InvalidXmlDefinition("version attribute is mandatory for element "+TAG);
	}
	public ComponentRefModel clone() {
		ComponentRefModel cr = new ComponentRefModel(TAG);
		cr.id=id;
		cr.version=version;
		return cr;
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getVersion() {
		return version;
	}

	public void setVersion(String version) {
		this.version = version;
	}
	public String toString() {
		return id+" ("+version+")";
	}
	public boolean equals(ComponentRefModel crm) { return crm.getId().equals(id); }
	public boolean equals(ComponentModel crm) { return crm.getId().equals(id); }
	public boolean equals(String id) { return this.id.equals(id); }
}
