/*
 * Copyright
 *   2007 axYus - www.axyus.com
 *   2005 C.Marchand - christophe.marchand@axyus.com
 *
 * This file is part of XEMELIOS.
 *
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package fr.gouv.finances.cp.xemelios.updater.config;

import java.util.ArrayList;
import java.util.Iterator;

import org.xml.sax.SAXException;

import fr.gouv.finances.cp.utils.xml.marshal.InvalidXmlDefinition;
import fr.gouv.finances.cp.utils.xml.marshal.XmlAttributes;
import fr.gouv.finances.cp.utils.xml.marshal.XmlMarshallable;
import fr.gouv.finances.cp.utils.xml.marshal.XmlOutputter;

public class FileListModel implements XmlMarshallable {
	public static final transient String TAG = "files";
	
	private String root;
	
	public ArrayList<XmlMarshallable> files;

	public FileListModel(String tag) {
		super();
		files=new ArrayList<XmlMarshallable>();
	}

	public void addCharacterData(String cData) throws SAXException {
	}

	public void addChild(XmlMarshallable child, String tagName) throws SAXException {
		files.add(child);
	}

	public XmlMarshallable getAttributes(XmlAttributes attributes) throws SAXException {
		root = attributes.getValue("root");
		return this;
	}

	public void marshall(XmlOutputter output) {
		output.startTag(TAG);
		output.addAttribute("root",root);
		for(Iterator<XmlMarshallable> it=files.iterator();it.hasNext();)
			it.next().marshall(output);
		output.endTag(TAG);
	}

	public void validate() throws InvalidXmlDefinition { }
	public FileListModel clone() {
		FileListModel flm = new FileListModel(TAG);
		flm.root=root;
		for(Iterator<XmlMarshallable> it=files.iterator();it.hasNext();)
			flm.files.add((XmlMarshallable)it.next().clone());
		return flm;
	}

	public ArrayList<XmlMarshallable> getFiles() {
		return files;
	}

	public void setFiles(ArrayList<XmlMarshallable> files) {
		this.files = files;
	}

	public String getRoot() {
		return root;
	}

	public void setRoot(String root) {
		this.root = root;
	}

}
